#//-----------------------------------------------
#//TUSB-KxxRL用ドライバアクセス用API
#//Python 用モジュール
#//
#//2024_04_10　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

from ctypes import *

#Device DLL Loader ( cdecl )
tusbkrldll = cdll.LoadLibrary("TUSBKRL.DLL")

tusbkrldll.TUSBKRL_Device_Open.restype = c_short
tusbkrldll.TUSBKRL_Device_Open.argtypes = (c_short,)
tusbkrldll.TUSBKRL_Device_Close.restype = None
tusbkrldll.TUSBKRL_Device_Close.argtypes = (c_short,)
tusbkrldll.TUSBKRL_Set.restype = c_short
tusbkrldll.TUSBKRL_Set.argtypes = (c_short,c_ubyte)
tusbkrldll.TUSBKRL_BitSet.restype = c_short
tusbkrldll.TUSBKRL_BitSet.argtypes = (c_short,c_ubyte,c_ubyte)
tusbkrldll.TUSBKRL_Read.restype = c_short
tusbkrldll.TUSBKRL_Read.argtypes = (c_short,POINTER(c_ubyte))

#######  support functions (don not call directry from user codes)

def isint(n):
    return type(n) is int

def IsInRange(Val,Min,Max):
    if(isint(Val) == False ):   return False
    elif(Val < Min):            return False
    elif(Val > Max):            return False
    else:                       return True

def IsValidId(DevId):
    return IsInRange(DevId,0,15)

# def functions ------------------------------------
def TUSBKRL_Device_Open(DevId):
    """Open Device Connection
    
    Arguments:
    DevId           ----    Device ID 0~15

    Returns:
    int             ----    Result code
    """
    if(not IsValidId(DevId)): return 8 # Invalid parameter
    return tusbkrldll.TUSBKRL_Device_Open(DevId)

def TUSBKRL_Device_Close(DevId):
    """Close Device Connection
    
    Arguments:
    DevId           ----    Device ID 0~15

    Returns:
    int             ----    Result code
    """
    if(not IsValidId(DevId)): return # Invalid parameter
    tusbkrldll.TUSBKRL_Device_Close(DevId)

def TUSBKRL_Set(DevId,Data):
    """Set On/Off Relays
    
    Arguments:
    DevId           ----    Device ID 0~15
    Data            ----    Output data bits

    Returns;
    int             ----    Result code
    """
    if(not IsValidId(DevId)): return 8 # Invalid parameter
    if(not IsInRange(Data,0,255)): return 8 # Invalid parameter
    return tusbkrldll.TUSBKRL_Set(DevId,Data)

def TUSBKRL_BitSet(DevId,Bit,Data):
    """Set On/Off one Relay
    
    Arguments:
    DevId           ----    Device ID 0~15
    Bit             ----    Relay Number 0~
    Data            ----    Output 0:Off(Reset) 1:On(Set)

    Returns;
    int             ----    Result code
    """
    if(not IsValidId(DevId)): return 8 # Invalid parameter
    if(not IsInRange(Bit,0,7)): return 8 # Invalid parameter
    if(not IsInRange(Data,0,1)): return 8 # Invalid parameter
    return tusbkrldll.TUSBKRL_BitSet(DevId,Bit,Data)

def TUSBKRL_Read(DevId):
    """Read Relay State
    
    Arguments:
    DevId           ----    Device ID 0~15
    
    Returns:
    int             ----    Result code
    int             ----    Output Status
    """
    if(not IsValidId(DevId)): return 8,None,None # Invalid parameter
    OUTStatus = c_ubyte()
    ret =  tusbkrldll.TUSBKRL_Read(DevId,byref(OUTStatus))
    if ret == 0:
        return 0,OUTStatus.value
    else:
        return ret,None
    
#戻り値用エラーメッセージ(オープン関数)
ErrStrDic = {
            0:"正常終了しました",
            1:"ID番号が不正です",
            2:"ドライバが初期化されていません",
            3:"すでにデバイスはオープンされています",
            4:"接続されている台数が多すぎます",
            5:"オープンできませんでした",
            6:"デバイスがみつかりません",
            8:"パラメータエラーです",
            9:"USB通信エラーです",
            99:"その他のエラーです"
}
def GetErrMessage(retcode):
    if( retcode in ErrStrDic):
        return ErrStrDic[retcode]
    else:
        return "不明なエラーです"
